package org.freehep.jas;

import java.awt.EventQueue;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.net.Authenticator;
import java.net.MalformedURLException;
import java.net.PasswordAuthentication;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;
import java.security.AllPermission;
import java.security.CodeSource;
import java.security.PermissionCollection;
import java.security.Permissions;
import java.security.Policy;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import javax.swing.JComponent;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import org.freehep.application.studio.Studio;
import org.freehep.jas.plugin.xmlio.XMLIOPlugin;
import org.freehep.jas.services.PreferencesTopic;
import org.freehep.jas.services.WebBrowser;
import org.freehep.util.commanddispatcher.CommandState;
import org.freehep.util.commandline.CommandLine;
import org.openide.util.Lookup.Item;
import org.openide.util.Lookup.Template;
import org.freehep.jas.util.waitcursor.WaitCursorEventQueue;
import org.freehep.xml.menus.XMLMenuBuilder;
import org.freehep.application.PropertyUtilities;
import org.freehep.application.studio.PluginInfo;
import org.freehep.graphicsbase.util.export.ExportFileType;

/**
 *
 * @author tonyj
 * @version $Id: JAS3.java 14098 2013-01-31 01:33:30Z onoprien $
 */
public class JAS3 extends Studio {

    private XMLIOPlugin xmlioPlugin;
    public static final File BUILTIN = new File("");

    private JAS3() {
        super("JAS3");

        // This is required so that unsigned plugins can be run when JAS
        // is running as a (signed) web-start app.
        Policy.setPolicy(new Policy() {

            @Override
            public PermissionCollection getPermissions(CodeSource codesource) {
                Permissions perms = new Permissions();
                perms.add(new AllPermission());
                return (perms);
            }

            @Override
            public void refresh() {
            }
        });
    }

    @Override
    protected CommandLine createCommandLine() {
        CommandLine cl = super.createCommandLine();
        // register standard options. There should be a more modular way to do
        // this!
        cl.addOption("startPage", "s", "url", "URL for the welcome page");
        cl.addOption("title", "t", "window title", "Title for the window");
        cl.addOption("open",null,"file","File to open");
        cl.addParameter("file...", "Files to be read in");
        return cl;
    }

    @Override
    protected void init() {
        try {
            // Register a general purpose URLStreamHandlerFactory
            URL.setURLStreamHandlerFactory(new JAS3StreamHandlerFactory());
            // Register a general purpose Authenticator
            Authenticator.setDefault(new JAS3Authenticator());
            URLConnection.setDefaultAllowUserInteraction(true);

            getLookup().add(new Preferences());
            setStatusMessage("Loading built-in modules...");
            InputStream in = JAS3.class.getResourceAsStream("built-in.xml");
            if (in != null) {
                List<PluginInfo> plugins = buildPluginList(in, BUILTIN);
                loadPlugins(plugins, JAS3.class.getClassLoader());
            }

            // Try loading extensions found by the default ClassLoader
            // TODO: Maybe we should always use this scheme for loading built-in plugins

            Enumeration e = JAS3.class.getClassLoader().getResources("PLUGIN-inf/plugins.xml");
            if (e.hasMoreElements()) {
                setStatusMessage("Loading extensions...");
                List<PluginInfo> sysPlugins = new ArrayList<PluginInfo>();
                while (e.hasMoreElements()) {
                    URL url = (URL) e.nextElement();
                    InputStream inx = url.openStream();
                    sysPlugins.addAll(buildPluginList(inx, BUILTIN));
                }
                loadPlugins(sysPlugins, JAS3.class.getClassLoader());
            }

            super.init();

            int cursorDelay = PropertyUtilities.getInteger(getUserProperties(), "waitCursorDelay", 250);
            if (cursorDelay > 0) {
                EventQueue waitQueue = new WaitCursorEventQueue(cursorDelay);
                Toolkit.getDefaultToolkit().getSystemEventQueue().push(waitQueue);
            }

            // Fix for JAS-85
            ExportFileType.setClassLoader(getExtensionLoader());
            // Fix for WIRED-408
            XMLMenuBuilder.setClassLoader(getExtensionLoader());

            String title = getCommandLine().getOption("title");
            if (title != null) {
                getAppProperties().setProperty("title", title);
            }

            xmlioPlugin = (XMLIOPlugin) getLookup().lookup(XMLIOPlugin.class);
            if (xmlioPlugin != null) {
                SwingUtilities.invokeLater(new Runnable() {

                    @Override
                    public void run() {
                        xmlioPlugin.restoreAtStart();
                    }
                });
            }
        } catch (IOException x) {
            throw new RuntimeException(x);
        }
    }

    @Override
    public String getSystemExtensionsDir() {
        if (System.getProperty("application.home") == null) {
            return null;
        } else {
            return super.getSystemExtensionsDir();
        }
    }

    @Override
    protected void saveUserProperties() {
        super.saveUserProperties();
        if (xmlioPlugin != null) {
            boolean success = xmlioPlugin.saveAtExit();
        }
    }

    public void onViewDocumentation() throws MalformedURLException {
        showWebPage("Documentation.URL");
    }

    public void enableViewDocumentation(CommandState state) {
        hasWebBrowser(state);
    }

    public void onViewForum() throws MalformedURLException {
        showWebPage("Forum.URL");
    }

    public void enableViewForum(CommandState state) {
        hasWebBrowser(state);
    }

    public void onReportBug() throws MalformedURLException {
        showWebPage("BugReport.URL");
    }

    public void enableReportBug(CommandState state) {
        hasWebBrowser(state);
    }

    public void onExamples() throws MalformedURLException {
        WebBrowser webBrowser = ((WebBrowser) getLookup().lookup(WebBrowser.class));
        webBrowser.showURL(new URL("classpath:/org/freehep/jas/web/examples.html"));
    }

    public void enableExamples(CommandState state) {
        hasWebBrowser(state);
    }

    private void hasWebBrowser(CommandState state) {
        state.setEnabled(getLookup().lookup(WebBrowser.class) != null);
    }

    private void showWebPage(String property) throws MalformedURLException {
        WebBrowser webBrowser = ((WebBrowser) getLookup().lookup(WebBrowser.class));
        webBrowser.showURL(new URL(getUserProperties().getProperty(property)), true);
    }

    public static void main(String[] args) {
        new JAS3().createFrame(args).setVisible(true);
    }

    @Override
    public void addToolBar(JToolBar toolBar, String name, int mode) {
        toolBar.setRollover(true);
        super.addToolBar(toolBar, name, mode);
    }

    private class Preferences implements PreferencesTopic {

        @Override
        public boolean apply(JComponent panel) {
            return ((GeneralPreferences) panel).apply();
        }

        @Override
        public JComponent component() {
            return new GeneralPreferences(JAS3.this);
        }

        @Override
        public String[] path() {
            return new String[]{"General"};
        }
    }

    private class JAS3StreamHandlerFactory implements URLStreamHandlerFactory {

        @Override
        public URLStreamHandler createURLStreamHandler(String protocol) {
            // The special cases are required for efficiency, and to avoid circular
            // dependencies when running in WebStart
            if ("http".equals(protocol)) {
                return null;
            }
            if ("jar".equals(protocol)) {
                return null;
            }
            if ("file".equals(protocol)) {
                return null;
            }
            if ("ftp".equals(protocol)) {
                return null;
            }

            Template template = new Template(URLStreamHandler.class, protocol, null);
            Item item = getLookup().lookupItem(template);
            return item == null ? null : (URLStreamHandler) item.getInstance();
        }
    }

    private class JAS3Authenticator extends Authenticator {

        private String[] fields = {"requestingHost",
            "requestingSite",
            "requestingPort",
            "requestingProtocol",
            "requestingPrompt",
            "requestingScheme"
        };

        @Override
        protected PasswordAuthentication getPasswordAuthentication() {
            try {
                String protocol = getRequestingProtocol();
                Template template = new Template(Authenticator.class, protocol, null);
                Item item = getLookup().lookupItem(template);
                if (item == null) {
                    return null;
                }
                Authenticator auth = (Authenticator) item.getInstance();

                // This is really ugly, but there is no other way to deal with Authenticator's stupid design.
                for (int i = 0; i < fields.length; i++) {
                    Field f = Authenticator.class.getDeclaredField(fields[i]);
                    f.setAccessible(true);
                    f.set(auth, f.get(this));
                }
                Method method = auth.getClass().getDeclaredMethod("getPasswordAuthentication", (Class[]) null);
                method.setAccessible(true);
                return (PasswordAuthentication) method.invoke(auth, (Object[]) null);
            } catch (Throwable t) {
                t.printStackTrace();
                return null;
            }
        }
    }
}
