package org.freehep.jas.plugin.pluginmanager;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import org.freehep.application.Application;
import org.freehep.application.studio.Plugin;
import org.freehep.application.studio.PluginInfo;
import org.freehep.application.studio.Studio;
import org.freehep.jas.JAS3;
import org.freehep.swing.ErrorDialog.ErrorDetailsDialog;
import org.freehep.util.images.ImageHandler;

class InstalledPluginPanel extends JPanel {

    private JButton remove;
    private JButton update;
    private JTree tree;
    private PluginInfoPanel infoPanel;
    private PluginActivationPanel activation;
    private PluginManager manager;
    static final Logger logger = Logger.getLogger(InstalledPluginPanel.class.getName());
    private ActionListener al = new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();
            if (source == remove) {
                try {
                    removeSelectedPlugins();
                } catch (Throwable io) {
                    logger.log(Level.SEVERE,"Error while removing plugin",io);
                }
            } else if (source == update) {
                manager.updateInstalledPlugins(InstalledPluginPanel.this);
            }
        }
    };
    private ChangeListener cl = new ChangeListener() {
        @Override
        public void stateChanged(ChangeEvent e) {
            updateTree();
        }
    };

    InstalledPluginPanel(PluginManager manager) {
        super(new BorderLayout());
        this.manager = manager;

        JPanel panel = new JPanel(new GridLayout(1, 2, 5, 5));
        tree = new JTree();
        tree.setCellRenderer(new Renderer());
        tree.setRootVisible(false);
        tree.setVisibleRowCount(8);
        tree.setShowsRootHandles(true); // For people who can't double-click
        tree.getSelectionModel().addTreeSelectionListener(new TreeHandler());

        JScrollPane treescroll = new JScrollPane(tree);
        treescroll.setBorder(BorderFactory.createTitledBorder("Installed Plugins"));
        panel.add(treescroll);

        JPanel rightPanel = new JPanel(new BorderLayout());
        infoPanel = new PluginInfoPanel() {
            private JLabel updateLabel;

            @Override
            void addExtraInfo(Object c1, Object c2) {
                add(new JLabel("Latest Version:"), c1);
                add(updateLabel = new JLabel(), c2);
            }

            @Override
            void setExtraInfo(PluginInfo info) {
                String label = null;
                Map<PluginInfo, PluginInfo> updateMap = InstalledPluginPanel.this.manager.getUpdateMap();
                if (updateMap != null) {
                    PluginInfo update = updateMap.get(info);
                    if (update != null) {
                        label = update.getVersion();
                    }
                }
                updateLabel.setText(label);
            }
        };
        infoPanel.setBorder(BorderFactory.createTitledBorder("Plugin Info"));
        rightPanel.add(infoPanel, BorderLayout.CENTER);

        activation = new PluginActivationPanel();
        activation.setBorder(BorderFactory.createTitledBorder("Plugin Activation"));
        rightPanel.add(activation, BorderLayout.SOUTH);

        panel.add(rightPanel);

        JPanel buttons = new JPanel();
        remove = new JButton("Remove selected plugins");
        remove.addActionListener(al);
        buttons.add(remove);
        update = new JButton("Update installed plugins");
        update.addActionListener(al);
        buttons.add(update);

        add(BorderLayout.SOUTH, buttons);
        add(BorderLayout.CENTER, panel);

        updateTree();
    }

    @Override
    public void addNotify() {
        super.addNotify();
        manager.addChangeListener(cl);
    }

    @Override
    public void removeNotify() {
        manager.removeChangeListener(cl);
        super.removeNotify();
    }

    private static DefaultMutableTreeNode addSubNode(DefaultMutableTreeNode root, Map<Object,DefaultMutableTreeNode> map, String name, Object dir) {
        if (dir != null) {
            if (dir instanceof String) {
                dir = new File((String) dir);
            }
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(name, true);
            map.put(dir, node);
            root.add(node);
            return node;
        }
        return null;
    }

    protected final void updateTree() {
        List<PluginInfo> loadedplugins = new ArrayList(manager.getPlugins());
        Comparator comparator = new PluginComparator();
        Collections.sort(loadedplugins, comparator);

        Studio app = manager.getApplication();
        DefaultMutableTreeNode treeRoot = new DefaultMutableTreeNode();
        Map<Object,DefaultMutableTreeNode> map = new HashMap<Object,DefaultMutableTreeNode>();
        addSubNode(treeRoot, map, "built-in", JAS3.BUILTIN);
        addSubNode(treeRoot, map, "system", app.getSystemExtensionsDir());
        addSubNode(treeRoot, map, "group", app.getGroupExtensionsDir());
        DefaultMutableTreeNode user = addSubNode(treeRoot, map, "user", app.getUserExtensionsDir());
        for (PluginInfo info : loadedplugins) {
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(info, false);
            DefaultMutableTreeNode rootNode = map.get(info.getLoadDirectory());
            rootNode.add(node);
        }

        DefaultTreeModel treeModel = new DefaultTreeModel(treeRoot, true);
        tree.setModel(treeModel);
        tree.expandPath(new TreePath(new Object[]{treeRoot, user}));

        remove.setEnabled(false);
        List<PluginInfo> updates = manager.getUpdatablePlugins();
        update.setEnabled(updates != null && !updates.isEmpty());
    }

    private void removeSelectedPlugins() throws IOException {
        TreePath[] selectednodes = tree.getSelectionPaths();
        // The jar files are probably locked by the ClassLoader, so
        // instead of trying to remove them we just write a list of files
        // to be removed. The actual removal is done next time JAS starts

        if (selectednodes != null && selectednodes.length > 0) {
            List<File> toBeDeleted = new ArrayList<File>();
            for (int i = 0; i < selectednodes.length; i++) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) selectednodes[i].getLastPathComponent();

                PluginInfo plugleaf = (PluginInfo) node.getUserObject();
                File dir = plugleaf.getLoadDirectory();
                Map<String, String> files = plugleaf.getFiles();
                for (String fileName :  files.keySet()) {
                    File removefile = new File(dir, fileName);
                    File parent = removefile.getParentFile();
                    if (!parent.canWrite()) {
                        throw new RuntimeException("File " + removefile + " cannot be removed");
                    }
                    toBeDeleted.add(removefile);
                }
                manager.removePlugin(plugleaf);
            }
            for (File toBeRemoved : toBeDeleted) {
                OutputStream out = new FileOutputStream(toBeRemoved);
                out.close();
            }
            updateTree();
            JOptionPane.showMessageDialog(this, "You must restart JAS for these changes to take effect");
        }
    }

    private class PluginActivationPanel extends JPanel implements ActionListener {

        /**
         * Creates new form PluginActivationPanel
         */
        PluginActivationPanel() {
            initComponents();
            setPlugin(null);
        }

        final void setPlugin(PluginInfo info) {
            this.info = info;
            setEnable();
        }

        private void setEnable() {
            if (info == null) {
                startCheckBox.setEnabled(false);
                startButton.setEnabled(false);
                stopButton.setEnabled(false);
                errorButton.setEnabled(false);
            } else {
                startCheckBox.setEnabled(info.hasMainClass());
                startCheckBox.setSelected(info.isLoadAtStart());
                Plugin plugin = info.getPlugin();
                startButton.setEnabled(plugin == null && info.hasMainClass());
                stopButton.setEnabled(plugin != null && plugin.canBeShutDown());
                errorButton.setEnabled(info.getErrorStatus() != null);
            }
        }

        /**
         * This method is called from within the constructor to initialize the
         * form.
         */
        private void initComponents() {
            GridBagConstraints gridBagConstraints;

            startCheckBox = new JCheckBox();
            startButton = new JButton();
            stopButton = new JButton();
            errorButton = new JButton();

            setLayout(new GridBagLayout());

            startCheckBox.setText("Start when application loads");
            startCheckBox.addActionListener(this);

            gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
            gridBagConstraints.anchor = GridBagConstraints.WEST;
            add(startCheckBox, gridBagConstraints);

            startButton.setText("Start");
            startButton.addActionListener(this);

            gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.insets = new Insets(0, 20, 0, 0);
            add(startButton, gridBagConstraints);

            stopButton.setText("Stop");
            stopButton.addActionListener(this);
            gridBagConstraints.insets = new Insets(0, 5, 0, 0);
            add(stopButton, gridBagConstraints);

            errorButton.setText("Show Error...");
            errorButton.addActionListener(this);
            add(errorButton, gridBagConstraints);
        }

        private void startCheckBoxActionPerformed(ActionEvent evt) {
            info.setLoadAtStart(startCheckBox.isSelected());
        }

        private void stopButtonActionPerformed(ActionEvent evt) {
            manager.getApplication().stopPlugin(info);
            setEnable();
            InstalledPluginPanel.this.repaint();
        }

        private void startButtonActionPerformed(ActionEvent evt) {
            try {
                manager.getApplication().startPlugin(info);
                setEnable();
                InstalledPluginPanel.this.repaint();
            } catch (Throwable t) {
                JDialog parent = (JDialog) SwingUtilities.getAncestorOfClass(JDialog.class, this);
                Studio.error(parent, "Error starting plugin", t);
            }
        }

        private void errorButtonActionPerformed(ActionEvent evt) {
            JDialog parent = (JDialog) SwingUtilities.getAncestorOfClass(JDialog.class, this);
            ErrorDetailsDialog dlg = new ErrorDetailsDialog(parent, info.getErrorStatus());
            dlg.setDefaultCloseOperation(ErrorDetailsDialog.DISPOSE_ON_CLOSE);
            dlg.pack();
            dlg.setLocationRelativeTo(parent);
            dlg.setVisible(true);
        }

        @Override
        public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
            Object source = actionEvent.getSource();
            if (source == startCheckBox) {
                startCheckBoxActionPerformed(actionEvent);
            } else if (source == startButton) {
                startButtonActionPerformed(actionEvent);
            } else if (source == stopButton) {
                stopButtonActionPerformed(actionEvent);
            } else if (source == errorButton) {
                errorButtonActionPerformed(actionEvent);
            }
        }
        private PluginInfo info;
        // Variables declaration - do not modify
        private JButton startButton;
        private JCheckBox startCheckBox;
        private JButton stopButton;
        private JButton errorButton;
        // End of variables declaration
    }

    private class Renderer extends DefaultTreeCellRenderer {

        private Icon pluginIcon = ImageHandler.getIcon("icons/plugin.gif", InstalledPluginPanel.class);
        private Icon updateIcon = ImageHandler.getIcon("icons/plugin_update.gif", InstalledPluginPanel.class);
        private Icon disabledIcon = ImageHandler.getIcon("icons/plugin_disabled.gif", InstalledPluginPanel.class);
        private Icon errorIcon = ImageHandler.getIcon("icons/plugin_error.gif", InstalledPluginPanel.class);

        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
            if (value instanceof DefaultMutableTreeNode) {
                Object info = ((DefaultMutableTreeNode) value).getUserObject();
                if (info instanceof PluginInfo) {
                    PluginInfo plugleaf = (PluginInfo) info;
                    String label = plugleaf.getName();
                    Map updateMap = manager.getUpdateMap();
                    setIcon(pluginIcon);
                    if (updateMap != null) {
                        PluginInfo update = (PluginInfo) updateMap.get(info);
                        if ((update != null) && (Application.versionNumberCompare(update.getVersion(), plugleaf.getVersion()) > 0)) {
                            label = "<html><b>" + label + " (update available)</b>";
                            setIcon(updateIcon);
                        }
                    }
                    if (plugleaf.getErrorStatus() != null) {
                        setIcon(errorIcon);
                    } else if (plugleaf.getPlugin() == null) {
                        setIcon(disabledIcon);
                    }
                    setText(label);
                }
            }
            return this;
        }
    }

    private class TreeHandler implements TreeSelectionListener {

        @Override
        public void valueChanged(TreeSelectionEvent evt) {
            TreePath selection = evt.getPath();
            DefaultMutableTreeNode node = selection == null ? null : (DefaultMutableTreeNode) selection.getLastPathComponent();

            if (node != null && node.getUserObject() instanceof PluginInfo) {
                PluginInfo plugprops = (PluginInfo) node.getUserObject();
                infoPanel.setPlugin(plugprops);
                activation.setPlugin(plugprops);
            } else {
                infoPanel.setPlugin(null);
                activation.setPlugin(null);
            }

            boolean enableRemove = false;
            TreePath[] checkselection = tree.getSelectionPaths();
            if (checkselection != null) {
                enableRemove = true;

                Set writeableDirs = new HashSet();
                Studio app = manager.getApplication();
                String userDir = app.getUserExtensionsDir();
                checkDir(writeableDirs, app.getUserExtensionsDir());
                checkDir(writeableDirs, app.getGroupExtensionsDir());
                checkDir(writeableDirs, app.getSystemExtensionsDir());
                if (userDir != null) {
                    File dir = new File(userDir);
                    if (dir.canWrite()) {
                        writeableDirs.add(dir);
                    }
                }

                for (int i = 0; i < checkselection.length; i++) {
                    node = (DefaultMutableTreeNode) checkselection[i].getLastPathComponent();
                    if (!(node.getUserObject() instanceof PluginInfo)) {
                        enableRemove = false;
                        break;
                    } else {
                        PluginInfo info = (PluginInfo) node.getUserObject();
                        File dir = info.getLoadDirectory();
                        if (!writeableDirs.contains(dir)) {
                            enableRemove = false;
                            break;
                        }
                    }
                }
            }
            remove.setEnabled(enableRemove);
        }

        private void checkDir(Set writeableDirs, String dir) {
            if (dir != null) {
                File file = new File(dir);
                if (file.canWrite()) {
                    writeableDirs.add(file);
                }
            }
        }
    }
}