package org.freehep.jas.plugin.pluginmanager;

import java.awt.Component;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import org.freehep.application.Application;
import org.freehep.application.PropertyUtilities;
import org.freehep.application.studio.Plugin;
import org.freehep.application.studio.PluginInfo;
import org.freehep.application.studio.Studio;
import org.freehep.application.studio.ExtensionClassLoader;
import org.freehep.jas.services.PreferencesTopic;
import org.freehep.util.commanddispatcher.CommandProcessor;
import org.freehep.xml.menus.XMLMenuBuilder;
import org.freehep.xml.util.ClassPathEntityResolver;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import javax.swing.event.EventListenerList;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

/**
 * A Plugin which provides a Plugin Manager.
 *
 * @author tonyj
 * @version $Id: PluginManager.java 14095 2013-01-29 20:57:42Z tonyj $
 */
public class PluginManager extends Plugin implements Runnable, PreferencesTopic {

    private static String[] preferencesPath = {"Plugin Manager"};
    private boolean checkForPluginsAtStart;
    private List<PluginInfo> availablePlugins;
    private URL checkURL;
    private List<PluginInfo> installablePlugins;
    private List<PluginInfo>  updatablePlugins;
    private Map<PluginInfo,PluginInfo>  updateMap;
    private boolean pluginListIsReady;
    private Thread pluginListReaderThread;
    private EventListenerList listeners = new EventListenerList();
    private boolean dialogVisible = false;
    static final Logger logger = Logger.getLogger(PluginManager.class.getName());

    @Override
    protected void init() throws org.xml.sax.SAXException, java.io.IOException {
        final Studio app = getApplication();
        app.getLookup().add(this);
        Properties user = app.getUserProperties();
        String pluginURL = user.getProperty("PluginManager.URL");
        checkURL = pluginURL == null ? null : new URL(pluginURL);
        checkForPluginsAtStart = PropertyUtilities.getBoolean(user, "checkForPluginsAtStart", false);

        XMLMenuBuilder builder = app.getXMLMenuBuilder();
        java.net.URL xml = PluginManager.class.getResource("PluginManager.menus");
        builder.build(xml);

        app.getCommandTargetManager().add(new Commands());

        pluginListIsReady = false;
    }

    boolean isPluginListIsReady() {
        return pluginListIsReady;
    }

    List<PluginInfo> getAvailablePlugins() {
        return pluginListIsReady ? availablePlugins : null;
    }

    Map<PluginInfo,PluginInfo> getUpdateMap() {
        return pluginListIsReady ? updateMap : null;
    }

    void startPluginListDownload() {
        pluginListReaderThread = new Thread(this);
        pluginListReaderThread.setDaemon(true);
        pluginListReaderThread.setPriority(Thread.NORM_PRIORITY);
        pluginListReaderThread.start();
    }

    private String urlEncode(String in) throws UnsupportedEncodingException {
        return URLEncoder.encode(in, "UTF-8");
    }

    @Override
    public void run() {
        try {
            // We do an http post to the plugin URL, so we can send extra info
            // which may be useful to the receiver

            URLConnection connection = checkURL.openConnection();
            if (connection instanceof HttpURLConnection) {
                try {
                    Properties prop = getApplication().getAppProperties();

                    ByteArrayOutputStream byteStream = new ByteArrayOutputStream(512);
                    PrintWriter out = new PrintWriter(byteStream, true);
                    out.print("app.name=" + urlEncode(prop.getProperty("appName")));
                    out.print("&app.version=" + urlEncode(prop.getProperty("version")));
                    out.print("&os.name=" + urlEncode(System.getProperty("os.name")));
                    out.print("&os.arch=" + urlEncode(System.getProperty("os.arch")));
                    out.print("&os.version=" + urlEncode(System.getProperty("os.version")));
                    out.print("&java.version=" + urlEncode(System.getProperty("java.version")));
                    out.print("&java.vendor=" + urlEncode(System.getProperty("java.vendor")));
                    out.close();

                    String lengthString = String.valueOf(byteStream.size());
                    HttpURLConnection http = (HttpURLConnection) connection;
                    http.setRequestMethod("POST");
                    http.setUseCaches(false);
                    http.setDoOutput(true);
                    http.setRequestProperty("Content-Length", lengthString);
                    http.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
                    byteStream.writeTo(connection.getOutputStream());
                } catch (UnsupportedEncodingException x) {
                    logger.log(Level.WARNING, "Unable to request plugin list", x);
                }
            }

            // For the moment at least we will use JDOM here
            SAXBuilder builder = new SAXBuilder(true);
            builder.setEntityResolver(new ClassPathEntityResolver("plugin.dtd", Studio.class));
            java.io.InputStream in = connection.getInputStream();
            try {
                availablePlugins = new ArrayList<PluginInfo>();
                Document doc = builder.build(in);
                List<Element> root = doc.getRootElement().getChildren();
                for (Element node : root) {
                    PluginInfo plugin = new PluginInfo(node);
                    availablePlugins.add(plugin);
                }
                Runnable r = new Runnable() {
                    @Override
                    public void run() {
                        processAvailablePlugins(availablePlugins);
                        pluginListReaderThread = null;
                        fireStateChanged();
                    }
                };
                SwingUtilities.invokeLater(r);
            } finally {
                in.close();
            }
        } catch (Exception x) {
            logger.log(Level.WARNING, "Unable to download plugin list", x);
        }
    }

    /**
     * Getter for property checkAtStart.
     *
     * @return Value of property checkAtStart.
     */
    boolean isCheckAtStart() {
        return checkForPluginsAtStart;
    }

    /**
     * Setter for property checkAtStart.
     *
     * @param checkAtStart New value of property checkAtStart.
     */
    void setCheckAtStart(boolean checkAtStart) {
        this.checkForPluginsAtStart = checkAtStart;
        PropertyUtilities.setBoolean(getApplication().getUserProperties(), "checkForPluginsAtStart", checkAtStart);
    }

    /**
     * Getter for property checkURL.
     *
     * @return Value of property checkURL.
     */
    URL getCheckURL() {
        return this.checkURL;
    }

    /**
     * Setter for property checkURL.
     *
     * @param checkURL New value of property checkURL.
     */
    void setCheckURL(URL checkURL) {
        this.checkURL = checkURL;
        getApplication().getUserProperties().setProperty("PluginManager.URL", checkURL.toExternalForm());
    }

    @Override
    public boolean apply(JComponent panel) {
        return ((PluginPreferences) panel).apply();
    }

    @Override
    public JComponent component() {
        return new PluginPreferences(this);
    }

    @Override
    public String[] path() {
        return preferencesPath;
    }

    List<PluginInfo> getPlugins() {
        return getApplication().getPlugins();
    }

    List<PluginInfo> getInstallablePlugins() {
        return installablePlugins;
    }

    List<PluginInfo> getUpdatablePlugins() {
        return updatablePlugins;
    }

    void addChangeListener(ChangeListener l) {
        listeners.add(ChangeListener.class, l);
    }

    void removeChangeListener(ChangeListener l) {
        listeners.remove(ChangeListener.class, l);
    }

    private void fireStateChanged() {
        ChangeEvent event = new ChangeEvent(this);
        ChangeListener[] list = (ChangeListener[]) listeners.getListeners(ChangeListener.class);
        for (int i = 0; i < list.length; i++) {
            list[i].stateChanged(event);
        }
    }

    /**
     * Called once the list of available plugins are available
     */
    void processAvailablePlugins(List<PluginInfo> available) {
        // Loop over new plugins, and match them up with already existing
        // plugins
        Map<String,PluginInfo> loadedMap = new HashMap<String,PluginInfo>();
        java.util.List<PluginInfo> loadedplugins = getPlugins();
        for (PluginInfo info : loadedplugins) {
            loadedMap.put(info.getName(), info);
        }
        installablePlugins = new ArrayList<PluginInfo>();
        updatablePlugins = new ArrayList<PluginInfo>();
        updateMap = new HashMap<PluginInfo,PluginInfo>();
        for (PluginInfo info : available) {
            PluginInfo old = loadedMap.get(info.getName());
            if (old != null) {
                updateMap.put(old, info);
                if (Application.versionNumberCompare(info.getVersion(), old.getVersion()) > 0) {
                    updatablePlugins.add(info);
                }
            } else {
                installablePlugins.add(info);
            }
        }
        pluginListIsReady = true;
        // If the user asked to be notified of new plugins, and if the plugin manager is
        // not visible, let them know.
        boolean notify = PropertyUtilities.getBoolean(getApplication().getUserProperties(), "notifyPluginUpdates", true);
        if (!updatablePlugins.isEmpty() && !dialogVisible && notify) {
            Box message = Box.createVerticalBox();
            JLabel label = new JLabel("Updated plugins available");
            label.setAlignmentX(1.0f);
            message.add(label);
            JCheckBox ask = new JCheckBox("Don't show me this again");
            message.add(ask);
            String[] options = {"Install now", "Plugin Manager...", "Cancel"};
            int rc = JOptionPane.showOptionDialog(getApplication(), message, "Updates available", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE, null, options, options[0]);
            if (ask.isSelected()) {
                getApplication().getUserProperties().setProperty("notifyPluginUpdates", "false");
            }
            if (rc == 0) // install now
            {
                updateInstalledPlugins(getApplication());
            } else if (rc == 1) // plugin manager
            {
                showPluginManager();
            }
        }
    }

    private void showPluginManager() {
        if (!pluginListIsReady && pluginListReaderThread == null) {
            startPluginListDownload();
        }

        JFrame frame = (JFrame) SwingUtilities.getAncestorOfClass(JFrame.class, getApplication());
        JDialog dlg = new PluginManagerDialog(frame, PluginManager.this);
        dlg.setModal(true);
        dlg.setTitle("Plugin Manager");
        dlg.pack();
        dlg.setLocationRelativeTo(getApplication());
        try {
            dialogVisible = true;
            dlg.setVisible(true);
        } finally {
            dialogVisible = false;
        }
    }

    void updateInstalledPlugins(Component parent) {
        Map<File,String> downloads = new HashMap();
        String base = getApplication().getUserExtensionsDir(); // FixMe!
        for (PluginInfo info :  getUpdatablePlugins()) {
            Map<String,String> files = info.getFiles();
            for (Map.Entry<String,String> entry : files.entrySet()) {
                File key = new File(base, entry.getKey());
                downloads.put(key, entry.getValue());
            }
        }

        PluginDownload download = new PluginDownload(downloads);
        Thread t = new Thread(download);
        t.start();
        JOptionPane.showMessageDialog(parent, download, "Downloading...", JOptionPane.PLAIN_MESSAGE);

        Throwable status = download.getStatus();
        if (status != null) {
            JOptionPane.showMessageDialog(parent, "Download failed: " + status, "Download error", JOptionPane.ERROR_MESSAGE);
            download.cleanUp();
        } else {
            download.commit();

            // Add the new jars to the list of URL's
            JOptionPane.showMessageDialog(parent, "You must restart JAS for these changes to take effect");
        }
    }

    void removePlugin(PluginInfo plugleaf) {
        getApplication().getPlugins().remove(plugleaf);
        updatablePlugins.remove(plugleaf);
        availablePlugins.remove(plugleaf);
        fireStateChanged();
    }

    public void installPlugins(List downloadedPlugins, ExtensionClassLoader loader) {
        getApplication().loadPlugins(downloadedPlugins, loader);
        installablePlugins.removeAll(downloadedPlugins);
        fireStateChanged();
    }

    @Override
    protected void applicationVisible() {
        if (checkForPluginsAtStart && checkURL != null) {
            startPluginListDownload();
        }
    }

    class Commands extends CommandProcessor {

        public void onPluginManager() {
            showPluginManager();
        }
    }
}